"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GolangGenerator = void 0;
const templates = __importStar(require("./templates"));
const liquidjs_1 = require("liquidjs");
const graphql_1 = require("graphql");
class GolangGenerator {
    config;
    schema;
    _templateEngine;
    _templateOperation;
    types = {
        Int: 'Int',
        Float: 'Float',
        Boolean: 'Boolean',
        String: 'String',
        ID: 'ID',
    };
    scalars = {
        Int: 'int32',
        Float: 'float64',
        Boolean: 'bool',
        String: 'string',
        ID: 'string',
    };
    _enums = [];
    _inputs = [];
    _objects = [];
    constructor(schema, config) {
        this.config = config ?? {};
        this.config.generateHTTPClient ??= true;
        this.schema = schema;
        this._templateEngine = new liquidjs_1.Liquid({
            strictFilters: true,
            strictVariables: true,
        });
        this._templateOperation = this._templateEngine.parse(templates.GOLANG_OPERATION);
        (0, graphql_1.visit)((0, graphql_1.parse)((0, graphql_1.printSchema)(this.schema)), {
            ScalarTypeDefinition: node => {
                this.types[node.name.value] = this._formatName(node.name.value);
                this.scalars[node.name.value] = 'string';
                return false;
            },
            EnumTypeDefinition: node => {
                this.types[node.name.value] = this._formatName(node.name.value);
                this._enums.push(node);
                return false;
            },
            InputObjectTypeDefinition: node => {
                this.types[node.name.value] = this._formatName(node.name.value);
                this._inputs.push(node);
                return false;
            },
            ObjectTypeDefinition: node => {
                this.types[node.name.value] = this._formatName(node.name.value);
                this._objects.push(node);
                return false;
            },
        });
    }
    generate(documents) {
        const l = [...this._generatePackage(), ...this._generateImports()];
        if (this.config.generateHTTPClient) {
            l.push(templates.GOLANG_HTTP);
            documents?.forEach(document => l.push(...this._generateOperations(document)));
        }
        l.push(...this._generateSchema());
        return l.join('\n');
    }
    _formatName(name) {
        if (name.match(/^id$/i) || name.match(/^uuid$/i)) {
            return name.toUpperCase();
        }
        return name
            .replace(/(^_|_$)/, '')
            .split('_')
            .map(word => word[0]?.toUpperCase() + word.substr(1))
            .join('');
    }
    _generatePackage() {
        return [
            `package ${this.config.packageName ?? 'graphql'}`,
            '',
            '// Code generated by graphql-codegen-golang ; DO NOT EDIT.',
            '',
        ];
    }
    _generateImports() {
        if (this.config.generateHTTPClient) {
            return [
                'import (',
                '  "bytes"',
                '  "encoding/json"',
                '  "fmt"',
                '  "io/ioutil"',
                '  "net/http"',
                '  "strings"',
                ')',
                '',
            ];
        }
        return [];
    }
    _generateSection(name) {
        return ['', '//', `// ${name}`, '//', ''];
    }
    _generateSchema() {
        return [
            ...this._generateScalars(),
            ...this._generateEnums(),
            ...this._generateInputs(),
            ...this._generateObjects(),
        ];
    }
    _generateScalars() {
        const l = [...this._generateSection('Scalars')];
        Object.entries(this.scalars).map(([gqlType, goType]) => {
            l.push(`type ${this.types[gqlType]} ${goType}`);
        });
        return l;
    }
    _generateEnums() {
        const l = [...this._generateSection('Enums')];
        this._enums.forEach(node => {
            const goType = this.types[node.name.value];
            l.push('', `type ${this.types[node.name.value]} string`, 'const (');
            node.values?.forEach(value => {
                const name = value.name.value;
                l.push(`  ${goType}${this._formatName(name)} ${goType} = "${name}"`);
            });
            l.push(')');
        });
        return l;
    }
    _generateInputs() {
        const l = [...this._generateSection('Inputs')];
        this._inputs.forEach(node => {
            const goType = this.types[node.name.value];
            l.push('', `type ${goType} struct {`);
            node.fields?.forEach(field => {
                try {
                    l.push(this._generateField(field.name.value, field.type));
                }
                catch (e) {
                    if (e instanceof UnsupportedTypeError) {
                        console.debug(`Skipping ${e.message}`);
                    }
                    else {
                        throw e;
                    }
                }
            });
            l.push('}');
        });
        return l;
    }
    _generateObjects() {
        const l = [...this._generateSection('Objects')];
        this._objects.forEach(node => {
            const goType = this.types[node.name.value];
            l.push('', `type ${goType} struct {`);
            node.fields?.forEach(field => {
                try {
                    l.push(this._generateField(field.name.value, field.type));
                }
                catch (e) {
                    if (e instanceof UnsupportedTypeError) {
                        console.debug(`Skipping ${e.message}`);
                    }
                    else {
                        throw e;
                    }
                }
            });
            l.push('}');
        });
        return l;
    }
    _generateField(name, type) {
        return `  ${this._formatName(name)} ${this._generateFieldType(type, name)}`;
    }
    _generateFieldType(type, fieldName, prefix = '', nonNull = false) {
        if (type.kind === 'NamedType' && this.types[type.name.value]) {
            return [
                `${nonNull ? '' : '*'}${prefix}`,
                `${this.types[type.name.value]} `,
                `\`json:"${fieldName}${nonNull ? '' : ',omitempty'}"\``,
            ].join('');
        }
        if (type.kind === 'NonNullType') {
            return this._generateFieldType(type.type, fieldName, prefix, prefix !== '' ? false : true);
        }
        if (type.kind === 'ListType') {
            return this._generateFieldType(type.type, fieldName, prefix + '[]', nonNull);
        }
        throw new UnsupportedTypeError(`unsupported field type "${type.name.value}"`);
    }
    _generateOperations(document) {
        const l = [];
        const typeInfo = new graphql_1.TypeInfo(this.schema);
        (0, graphql_1.visit)(document, (0, graphql_1.visitWithTypeInfo)(typeInfo, {
            OperationDefinition: {
                enter: operation => {
                    if (!operation.name)
                        return false;
                    if (operation.operation === 'subscription')
                        return false;
                    const name = this._formatName(operation.name.value);
                    l.push(...this._generateSection(`${(0, graphql_1.print)(operation).split('{', 1)[0]?.trim()}`));
                    l.push(...this._generateOperationVariables(name, operation.variableDefinitions));
                    l.push(`type ${name}Response struct {`);
                    return;
                },
                leave: operation => {
                    if (!operation.name)
                        return;
                    l.push('}', '');
                    const name = this._formatName(operation.name.value);
                    const hasVariables = this._generateOperationVariables(name, operation.variableDefinitions).length > 0;
                    l.push(...this._generateOperationCode(name, (0, graphql_1.print)(operation), hasVariables));
                },
            },
            Field: {
                enter: field => {
                    const name = this._formatName(field.name.value);
                    const w = [`  ${name} `];
                    if (field.selectionSet) {
                        const outputType = typeInfo
                            .getType()
                            ?.toString();
                        if (outputType?.startsWith('[')) {
                            if (!outputType.endsWith('!')) {
                                w.push('*');
                            }
                            w.push('[]');
                        }
                        w.push('struct {');
                    }
                    else {
                        w.push(`string \`json:"${field.name.value}"\``);
                    }
                    l.push(w.join(''));
                },
                leave: field => {
                    if (field.selectionSet) {
                        l.push(`} \`json:"${field.name.value}"\``);
                    }
                },
            },
        }));
        return l;
    }
    _generateOperationVariables(name, variableDefinitions) {
        if (!variableDefinitions || variableDefinitions.length === 0) {
            return [];
        }
        const l = [`type ${name}Variables struct {`];
        variableDefinitions.forEach(variable => {
            try {
                l.push(this._generateField(variable.variable.name.value, variable.type));
            }
            catch (e) {
                if (e instanceof UnsupportedTypeError) {
                    console.debug(`Skipping ${e.message}`);
                }
                else {
                    throw e;
                }
            }
        });
        l.push('}', '');
        return l;
    }
    _generateOperationCode(name, operation, hasVariables) {
        const l = [];
        l.push(this._templateEngine.renderSync(this._templateOperation, {
            name,
            operation,
            hasVariables,
        }));
        return l;
    }
}
exports.GolangGenerator = GolangGenerator;
class UnsupportedTypeError extends Error {
    constructor(msg) {
        super(msg);
        Object.setPrototypeOf(this, UnsupportedTypeError.prototype);
    }
}
//# sourceMappingURL=generator.js.map